/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collection;
import java.util.LinkedList;

import org.junit.Assert;
import org.junit.Before;

import com.bitctrl.Constants;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dua.abfrpuffer.Ereignis.EreignissTyp;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.bitctrl.dua.DUAKonstanten;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Testet dem Abfragemodul.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class AbfragePufferDatenTester extends AbfragePufferdaten {

	private static final Debug LOGGER = Debug.getLogger();
	/**
	 * Liste der SM die simuliert werden.
	 */
	private final Collection<SteuerModulSim> steuerModuleSim = new LinkedList<>();
	/**
	 * Length des statischen Arrays.
	 */
	private final int paramLen = 5;
	/**
	 * Aktueller index in dem Ringpuffer.
	 */
	private int paramIndex;
	/**
	 * Ringpuffer fuer axl abfrageModul Parametrierung, Offset der axl Abrufe.
	 */
	private final long[] axlOffset = new long[paramLen];
	/**
	 * Ringpuffer fuer axl abfrageModul Parametrierung, Zyklus der axl Abrufe.
	 */
	private final long[] axlZyklus = new long[paramLen];
	/**
	 * Ringpuffer fuer axl abfrageModul Parametrierung, Offset der lve Abrufe.
	 */
	private final long[] lveOffset = new long[paramLen];
	/**
	 * Ringpuffer fuer axl abfrageModul Parametrierung, Zyklus der lve Abrufe.
	 */
	private final long[] lveZyklus = new long[paramLen];
	/**
	 * Datenbeschreibung fuer die Sendung von Abfragemodulparameter.
	 */
	private DataDescription ddAbfrageModulSend;

	/**
	 * Vorbereitet den Status fuer die Tests.
	 *
	 * @throws Exception
	 *             Fehler
	 */
	@Before
	public void prepare() throws Exception {

		final int lveIntervall = 1;
		final int axlIntervall = 1;

		final LinkedList<DePid> list = new LinkedList<>();
		/*
		 * list.clear(); list.add(new dePid("testLVE.SM_1001.EAK1.DeLve_1",
		 * "fs.test.1.2.1", lveIntervall)); list.add(new
		 * dePid("testLVE.SM_1001.EAK1.DeLve_2", "fs.test.1.2.2",
		 * lveIntervall)); list.add(new dePid("testLVE.SM_1001.EAK1.DeLve_3",
		 * "fs.test.1.2.3", lveIntervall));
		 *
		 * ////////////////////////////// // SteuerModul testLVE.SM_1001
		 * ///////////////////////////// steuerModuleSim.add(new
		 * SteuerModulSim(null, null, list,
		 * "testLVE.SM_1001.EAK1.LveDeSammelkanal_1"));
		 *
		 * list.clear(); list.add(new dePid("testLVE.SM_1002.EAK1.DeLve_1",
		 * "fs.test.2.2.1", lveIntervall)); list.add(new
		 * dePid("testLVE.SM_1002.EAK1.DeLve_2", "fs.test.2.2.2",
		 * lveIntervall)); list.add(new dePid("testLVE.SM_1002.EAK1.DeLve_3",
		 * "fs.test.2.2.3", lveIntervall));
		 *
		 * ////////////////////////////// // SteuerModul testLVE.SM_1002
		 * ///////////////////////////// steuerModuleSim.add(new
		 * SteuerModulSim(null, null, list,
		 * "testLVE.SM_1002.EAK1.LveDeSammelkanal_1"));
		 *
		 *
		 * list.clear(); list.add(new dePid("testLVE.SM_1003.EAK1.DeLve_1",
		 * "fs.test.3.2.1", lveIntervall)); list.add(new
		 * dePid("testLVE.SM_1003.EAK1.DeLve_2", "fs.test.3.2.2",
		 * lveIntervall)); list.add(new dePid("testLVE.SM_1003.EAK1.DeLve_3",
		 * "fs.test.3.2.3", lveIntervall));
		 *
		 * ////////////////////////////// // SteuerModul testLVE.SM_1003
		 * ///////////////////////////// steuerModuleSim.add(new
		 * SteuerModulSim(null, null, list,
		 * "testLVE.SM_1003.EAK1.LveDeSammelkanal_1"));
		 */

		list.clear();
		list.add(new DePid("testLVE.SM_1004.EAK1.DeLve_1", "fs.test.4.2.1", lveIntervall));
		list.add(new DePid("testLVE.SM_1004.EAK1.DeLve_2", "fs.test.4.2.2", lveIntervall));
		list.add(new DePid("testLVE.SM_1004.EAK1.DeLve_3", "fs.test.4.2.3", lveIntervall));

		// ////////////////////////////
		// SteuerModul testLVE.SM_1004
		// ///////////////////////////
		steuerModuleSim.add(new SteuerModulSim(null, null, list, "testLVE.SM_1004.EAK1.LveDeSammelkanal_1"));

		list.clear();
		list.add(new DePid("testAXL.SM_1001.EAK1.DeAxl_1", "testAXL.SM_1001.EAK1.MsAxl_1", axlIntervall));
		list.add(new DePid("testAXL.SM_1001.EAK1.DeAxl_2", "testAXL.SM_1001.EAK1.MsAxl_2", axlIntervall));
		list.add(new DePid("testAXL.SM_1001.EAK1.DeAxl_3", "testAXL.SM_1001.EAK1.MsAxl_3", axlIntervall));

		// ////////////////////////////
		// SteuerModul testAXL.SM_1001
		// ///////////////////////////
		steuerModuleSim.add(new SteuerModulSim(list, "testAXL.SM_1001.EAK1.AxlDeSammelkanal_1", null, null));

		// Die default Werte im DAV, sonst wird es dem Test nicht bestehen
		// Abfrage Modul muss so parametriert werden !
		axlOffset[paramLen - 1] = 1;
		axlZyklus[paramLen - 1] = 2;

		lveOffset[paramLen - 1] = 1;
		lveZyklus[paramLen - 1] = 2;
	}

	/**
	 * Einfaches Test um die AbrufIntervall und Offset Parametrierung
	 * Aenderungen zu Testen.
	 */
	// @Test
	public synchronized void abfrageIntervallAenderungTest() {
		try {

			// laeuft ohne SM, testet nur die AnfrageIntervalle
			steuerModuleSim.clear();
			ZeitSimulator.start("2006.01.01 4:00:00.000", 5000, 8000);
			StandardApplicationRunner.run(this, Verbindung.CON_DATA.clone());

			axlOffset[0] = 5;
			axlZyklus[0] = 1;

			lveOffset[0] = 8;
			lveZyklus[0] = 1;

			ZeitSimulator.schedule(new Ereignis(this, EreignissTyp.EREIGNIS_ABFRAGEMODUL_PARAM),
					"2006.01.01 8:30:00.000");

			axlOffset[1] = 15;
			axlZyklus[1] = 6;

			lveOffset[1] = 3;
			lveZyklus[1] = 4;

			ZeitSimulator.schedule(new Ereignis(this, EreignissTyp.EREIGNIS_ABFRAGEMODUL_PARAM),
					"2006.01.01 20:15:00.000");

			axlOffset[2] = 1;
			axlZyklus[2] = 2;

			lveOffset[2] = 1;
			lveZyklus[2] = 2;

			ZeitSimulator.schedule(new Ereignis(this, EreignissTyp.EREIGNIS_ABFRAGEMODUL_PARAM),
					"2006.01.02 06:00:00.000");

			ZeitSimulator.schedule(new Ereignis(this, EreignissTyp.EREIGNIS_SIMULATION_ENDE),
					"2006.01.02 12:00:00.000");
			this.wait();

		} catch (final Exception e) {
			getDav().disconnect(true, "Fehler bei Test: abfrageIntervallAenderungTest()");
			System.err.println(e.getMessage());
			e.printStackTrace();
		}

		getDav().disconnect(false, Constants.EMPTY_STRING);
	}

	/**
	 * Testfall, De beginnt zu flackern.
	 */
	// @Test
	public synchronized void flackerTest() {
		try {

			ZeitSimulator.start("2006.01.01 6:00:00.000", 20000, 8000);
			StandardApplicationRunner.run(this, Verbindung.CON_DATA.clone());

			ZeitSimulator.schedule(new Ereignis(SteuerModulSim.getDe("testLVE.SM_1004.EAK1.DeLve_1"),
					EreignissTyp.EREIGNIS_DE_FLACKERT), "2006.01.01 09:40:00.000");

			ZeitSimulator.schedule(new Ereignis(SteuerModulSim.getDe("testLVE.SM_1004.EAK1.DeLve_1"),
					EreignissTyp.EREIGNIS_DE_FLACKERT), "2006.01.01 10:25:00.000");

			ZeitSimulator.schedule(new Ereignis(this, EreignissTyp.EREIGNIS_SIMULATION_ENDE),
					"2006.01.01 15:00:00.000");
			this.wait();

		} catch (final Exception e) {
			getDav().disconnect(true, "Fehler bei Test: flackerTest()");
			System.err.println(e.getMessage());
			e.printStackTrace();
		}

		getDav().disconnect(false, Constants.EMPTY_STRING);
	}

	/**
	 * Testfall, die Parametrierung des Archivs aendert sich.
	 */
	// @Test
	public synchronized void parametrierungTest() {
		try {

			ZeitSimulator.start("2006.01.01 6:00:00.000", 5000, 8000);
			StandardApplicationRunner.run(this, Verbindung.CON_DATA.clone());

			ZeitSimulator.schedule(new Ereignis(SteuerModulSim.getDe("testLVE.SM_1004.EAK1.DeLve_1"),
					EreignissTyp.EREIGNIS_DE_WIRD_NICHT_ARCHIVIERT), "2006.01.01 09:30:00.000");

			ZeitSimulator.schedule(new Ereignis(SteuerModulSim.getDe("testLVE.SM_1004.EAK1.DeLve_1"),
					EreignissTyp.EREIGNIS_DE_WIRD_ARCHIVIERT), "2006.01.01 15:30:00.000");

			ZeitSimulator.schedule(new Ereignis(this, EreignissTyp.EREIGNIS_SIMULATION_ENDE),
					"2006.01.02 02:10:00.000");
			this.wait();

		} catch (final Exception e) {
			getDav().disconnect(true, "Fehler bei Test: parametrierungTest()");
			System.err.println(e.getMessage());
			e.printStackTrace();
		}

		getDav().disconnect(false, Constants.EMPTY_STRING);
	}

	/**
	 * Testfall, das SystemDe kann die Anfragen nicht verstehen, antwortete mit
	 * Negativer Quittung.
	 */
	// @Test
	public synchronized void runNegativeQuittungTest() {

		final DateFormat datumFormat = new SimpleDateFormat(DUAKonstanten.ZEIT_FORMAT_GENAU_STR);
		try {
			long zeit = datumFormat.parse("2006.01.01 20:00:00.000").getTime();
			SteuerModulSim.setAeltesterLveZeitStempellVorhanden(steuerModuleSim, zeit);

			zeit = datumFormat.parse("2006.01.01 20:00:00.000").getTime();
			SteuerModulSim.setAeltesterAxlZeitStempellVorhanden(steuerModuleSim, zeit);

			ZeitSimulator.start("2006.01.03 12:00:00.000", 5000, 8000);
			StandardApplicationRunner.run(this, Verbindung.CON_DATA.clone());

			ZeitSimulator.schedule(new Ereignis(SteuerModulSim.getDe("testLVE.SM_1004.EAK1.LveDeSammelkanal_1"),
					EreignissTyp.EREIGNIS_DE_KANN_ABFRAGE_NICHT_VERSTEHEN), "2006.01.03 17:30:00.000");

			ZeitSimulator.schedule(new Ereignis(SteuerModulSim.getDe("testLVE.SM_1004.EAK1.DeLve_1"),
					EreignissTyp.EREIGNIS_SM_OFFLINE), "2006.01.04 10:30:00.000");

			ZeitSimulator.schedule(new Ereignis(SteuerModulSim.getDe("testLVE.SM_1004.EAK1.DeLve_1"),
					EreignissTyp.EREIGNIS_DE_GEHT_ONLINE_MIT_DATEN), "2006.01.04 11:30:00.000");

			ZeitSimulator.schedule(new Ereignis(SteuerModulSim.getDe("testLVE.SM_1004.EAK1.LveDeSammelkanal_1"),
					EreignissTyp.EREIGNIS_DE_KANN_ABFRAGE_WIEDER_VERSTEHEN), "2006.01.04 12:30:00.000");

			ZeitSimulator.schedule(new Ereignis(this, EreignissTyp.EREIGNIS_KOSISTENZTEST), "2006.01.04 23:30:00.000");

			ZeitSimulator.schedule(new Ereignis(this, EreignissTyp.EREIGNIS_SIMULATION_ENDE),
					"2006.01.05 01:00:00.000");
			this.wait();

		} catch (final Exception e) {
			getDav().disconnect(true, "Fehler bei Test: runNegativeQuittungTest()");
			System.err.println(e.getMessage());
			e.printStackTrace();
		}

		getDav().disconnect(false, Constants.EMPTY_STRING);
	}

	/**
	 * Testfall, wenn ein De (nicht SystemDE ) kann die Anfragen nicht
	 * verstehen, antwortete mit Negativer Quittung.
	 */
	// @Test
	public synchronized void runNegativeQuittungEinfacheDeTest() {
		try {
			final DateFormat datumFormat = new SimpleDateFormat(DUAKonstanten.ZEIT_FORMAT_GENAU_STR);

			long zeit = datumFormat.parse("2006.01.01 20:00:00.000").getTime();
			SteuerModulSim.setAeltesterLveZeitStempellVorhanden(steuerModuleSim, zeit);

			zeit = datumFormat.parse("2006.01.01 20:00:00.000").getTime();
			SteuerModulSim.setAeltesterAxlZeitStempellVorhanden(steuerModuleSim, zeit);

			ZeitSimulator.start("2006.01.03 12:00:00.000", 5000, 8000);
			StandardApplicationRunner.run(this, Verbindung.CON_DATA.clone());

			ZeitSimulator.schedule(new Ereignis(SteuerModulSim.getDe("testLVE.SM_1004.EAK1.DeLve_1"),
					EreignissTyp.EREIGNIS_DE_KANN_ABFRAGE_NICHT_VERSTEHEN), "2006.01.03 17:30:00.000");

			ZeitSimulator.schedule(new Ereignis(SteuerModulSim.getDe("testLVE.SM_1004.EAK1.DeLve_1"),
					EreignissTyp.EREIGNIS_DE_KANN_ABFRAGE_WIEDER_VERSTEHEN), "2006.01.04 12:30:00.000");

			ZeitSimulator.schedule(new Ereignis(this, EreignissTyp.EREIGNIS_KOSISTENZTEST), "2006.01.04 23:30:00.000");

			ZeitSimulator.schedule(new Ereignis(this, EreignissTyp.EREIGNIS_SIMULATION_ENDE),
					"2006.01.05 01:00:00.000");
			this.wait();

		} catch (final Exception e) {
			getDav().disconnect(true, "Fehler bei Test: runNegativeQuittungEinfacheDeTest()");
			System.err.println(e.getMessage());
			e.printStackTrace();
		}

		getDav().disconnect(false, Constants.EMPTY_STRING);

	}

	/**
	 * Testfall, das DE kommot offline u online, mit und ohne Datenverlust.
	 */
	// @Test
	public synchronized void onlineOfflineTest() {
		try {
			final DateFormat datumFormat = new SimpleDateFormat(DUAKonstanten.ZEIT_FORMAT_GENAU_STR);

			final long zeit = datumFormat.parse("2006.01.02 12:00:00.000").getTime();
			SteuerModulSim.setAeltesterLveZeitStempellVorhanden(steuerModuleSim, zeit);
			SteuerModulSim.setAeltesterAxlZeitStempellVorhanden(steuerModuleSim, zeit);

			ZeitSimulator.start("2006.01.02 12:30:00.000", 5000, 8000);
			StandardApplicationRunner.run(this, Verbindung.CON_DATA.clone());

			ZeitSimulator.schedule(new Ereignis(SteuerModulSim.getDe("testLVE.SM_1004.EAK1.DeLve_2"),
					EreignissTyp.EREIGNIS_SM_OFFLINE), "2006.01.02 17:30:00.000");

			ZeitSimulator.schedule(new Ereignis(SteuerModulSim.getDe("testLVE.SM_1004.EAK1.DeLve_1"),
					EreignissTyp.EREIGNIS_SM_OFFLINE), "2006.01.02 19:30:00.000");

			ZeitSimulator.schedule(new Ereignis(SteuerModulSim.getDe("testLVE.SM_1004.EAK1.DeLve_2"),
					EreignissTyp.EREIGNIS_SM_GEHT_ONLINE), "2006.01.02 20:30:00.000");

			ZeitSimulator.schedule(new Ereignis(SteuerModulSim.getDe("testLVE.SM_1004.EAK1.DeLve_1"),
					EreignissTyp.EREIGNIS_DE_GEHT_ONLINE_MIT_DATEN), "2006.01.02 22:30:00.000");

			ZeitSimulator.schedule(new Ereignis(this, EreignissTyp.EREIGNIS_KOSISTENZTEST), "2006.01.03 01:30:00.000");

			ZeitSimulator.schedule(new Ereignis(this, EreignissTyp.EREIGNIS_SIMULATION_ENDE),
					"2006.01.03 03:30:00.000");
			this.wait();

		} catch (final Exception e) {
			getDav().disconnect(true, "Fehler bei Test: onlineOfflineTest()");
			System.err.println(e.getMessage());
			e.printStackTrace();
		}

		getDav().disconnect(false, Constants.EMPTY_STRING);
	}

	/**
	 * Testfall, das DE antwortet nicht mehr, es sollen Timeouts vorkommen.
	 */
	// @Test
	public synchronized void timeoutTest() {
		try {

			ZeitSimulator.start("2006.01.02 10:00:00.000", 5000, 8000);
			StandardApplicationRunner.run(this, Verbindung.CON_DATA.clone());

			ZeitSimulator.schedule(new Ereignis(SteuerModulSim.getDe("testLVE.SM_1004.EAK1.DeLve_1"),
					EreignissTyp.EREIGNIS_DE_ANWORTET_NICHT_MEHR), "2006.01.02 14:30:00.000");

			ZeitSimulator.schedule(new Ereignis(this, EreignissTyp.EREIGNIS_SIMULATION_ENDE),
					"2006.01.02 22:30:00.000");
			this.wait();

		} catch (final Exception e) {
			getDav().disconnect(true, "Fehler bei Test: timeoutTest()");
			System.err.println(e.getMessage());
			e.printStackTrace();
		}

		getDav().disconnect(false, Constants.EMPTY_STRING);
	}

	@Override
	public void anmeldeAbfrageModulParametrierung() {
		super.anmeldeAbfrageModulParametrierung();

		ddAbfrageModulSend = erzeugeDatenBeschreibung(AbfrPufferKonstanten.ATG_ABFRAGEMODUL,
				AbfrPufferKonstanten.ASP_PARAM_VORGABE, DataDescription.NO_SIMULATION_VARIANT_SET);

		try {
			getDav().subscribeSender(this, getDav().getDataModel().getObject("abfmodul.test.swe4.6"),
					ddAbfrageModulSend, SenderRole.sender());
		} catch (final Exception e) {
			AbfragePufferDatenTester.LOGGER.error("Anmeldung für AbfrageModul sender unmöglich:" + e.getMessage());
		}
		// System.out.println("DD " + DD_ABFRAGEMODUL_SEND);
	}

	/**
	 * Vergleicht, ob der Status des Simulationmodells dem Status des internen
	 * Models innerhalb des AbrageModuls entspricht.
	 */
	void testKonsistenz() {

		final long zeit = ZeitSimulator.getTimeMillis();

		for (final SteuerModulSim sm : steuerModuleSim) {

			// enthaelt keinen Sys Lve De
			if (sm.getSysLveDe() == null) {
				continue;
			}

			DatenEndgeraet deVerw = Station.getDatenEndgeraet(sm.getSysLveDe().getDePid());
			final SystemDatenEndgeraet sysDeVerw = (SystemDatenEndgeraet) deVerw;

			for (final DeSim deS : sm.getLveDeList()) {
				final LveDeSim deSim = (LveDeSim) deS;

				deVerw = Station.getDatenEndgeraet(deSim.getDePid());

				Assert.assertNotNull("DatenEndgeraet " + deSim.getDePid() + "ist im AbfrageModul null", deVerw);
				// if(deVerw
				// .getErzeugungsIntervall() != deSim
				// .getErzeugungIntervall()){
				// System.out.println("\n\n\n!!!!!!!!\n\n\n" + deVerw
				// .getErzeugungsIntervall() + ", " +
				// deSim.getErzeugungIntervall() + "\n\n\n!!!!!!!!\n\n\n");
				// }

				Assert.assertTrue(
						"Erzeugungsintervall nicht gleich: " + deVerw.getErzeugungsIntervall() + ", "
								+ deSim.getErzeugungIntervall(),
								Math.abs(deVerw.getErzeugungsIntervall()
										- deSim.getErzeugungIntervall()) < (Constants.MILLIS_PER_HOUR * 2));

				System.out.println("TestOK");

				// if (sysDeVerw.getStatus() !=
				// SystemDatenEndgeraet.SystemDeStatus.DE_STAT_ABFRAGE_LAEUFT) {
				// if(deVerw
				// .getErzeugungsIntervall() != deSim
				// .getErzeugungIntervall()){
				// System.out.println("\n\n\n!!!!!!!!\n\n\nLetzte Zeitstempell
				// nicht gleich\n"
				// + deVerw.getLetzterDatensatzZeitStempel() + ", " +
				// deSim.getLetzterZeitStempellGeschickt() +
				// "\n\n\n!!!!!!!!\n\n\n");
				// }

				// Assert.assertTrue("Letzter Zeitstempell nicht gleich: " +
				// deVerw.getLetzterDatensatzZeitStempel() + ", " + deSim
				// .getLetzterZeitStempellGeschickt(),
				// Math.abs(deVerw.getLetzterDatensatzZeitStempel() - deSim
				// .getLetzterZeitStempellGeschickt()) <
				// (Constants.MILLIS_PER_HOUR * 4));
				// }

			}

			if (sysDeVerw.getStatus() == SystemDatenEndgeraet.SystemDeStatus.DE_STAT_ABFRAGE_MOEGLICH) {
				for (final DatenEndgeraet de : sysDeVerw.getStation().getLveDeListe()) {
					Assert.assertTrue("Erwartet datensaetze obwohl abfrage moeglich ist",
							de.getAnzahlErwarteteDatenSaetze() == 0);
					Assert.assertTrue("Status nicht aktuell",
							(zeit - de.getLetzterDatensatzZeitStempel()) < de.getLetzterDatensatzZeitStempel());
				}
			}
		}
	}

	/**
	 * Sendet die Einstellungen fuer den AbfrageModul, Zeitperioden und Offsets.
	 */
	public void sendeAbfrageModulParametrierung() {

		final Data data = getDav()
				.createData(getDav().getDataModel().getAttributeGroup(AbfrPufferKonstanten.ATG_ABFRAGEMODUL));

		data.getUnscaledValue(AbfrPufferKonstanten.ATR_LVEABFRAGEZYKLUS).set(lveZyklus[paramIndex]);
		data.getUnscaledValue(AbfrPufferKonstanten.ATR_LVEABFRAGEOFFSET).set(lveOffset[paramIndex]);

		data.getUnscaledValue(AbfrPufferKonstanten.ATR_AXLABFRAGEZYKLUS).set(axlZyklus[paramIndex]);
		data.getUnscaledValue(AbfrPufferKonstanten.ATR_AXLABFRAGEOFFSET).set(axlOffset[paramIndex]);

		paramIndex = ++paramIndex % paramLen;

		final ResultData resDatei = new ResultData(getDav().getDataModel().getObject("abfmodul.test.swe4.6"),
				ddAbfrageModulSend, ZeitSimulator.getTimeMillis(), data);
		try {
			getDav().sendData(resDatei);
		} catch (final Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}

	@Override
	public void initialize(final ClientDavInterface connection) throws Exception {
		super.initialize(connection);

		// ShutdownHook wird beim beenden gerufen
		Runtime.getRuntime().addShutdownHook(new ShutdownHook());
		for (final SteuerModulSim sm : steuerModuleSim) {
			sm.initialize(connection);
		}
	}

	@Override
	public void parseArguments(final ArgumentList argumentList) throws Exception {
		super.parseArguments(argumentList);

		for (final SteuerModulSim sm : steuerModuleSim) {
			final ArgumentList argList = new ArgumentList(argumentList.getArgumentStrings());
			sm.parseArguments(argList);
		}

	}

	/**
	 * Damit bei Beendung der Applikation sich alle De's unmelden vom DAV.
	 */
	protected class ShutdownHook extends Thread {
		@Override
		public void run() {
			try {
				for (final SteuerModulSim sm : steuerModuleSim) {
					sm.unsubscribe();
				}
				disconnect();
			} catch (final Exception e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
			System.out.println("Quit");
		}
	}

	/**
	 * Abmeldung vom DAV.
	 */
	public void disconnect() {
		getDav().disconnect(false, "");
	}

	@Override
	void generiereRegelmaesigeLveAbfragen() {
		super.generiereRegelmaesigeLveAbfragen();

		// System.out.println("## LVE SCHEDULER @"
		// + DUAKonstanten.ZEIT_FORMAT_GENAU.format(new
		// Date(System.currentTimeMillis())) + ", Abfrageversuch: "
		// + DUAKonstanten.ZEIT_FORMAT_GENAU.format(new
		// Date(naechsteAbfrageLve)) + ", Virtuell: "
		// + DUAKonstanten.ZEIT_FORMAT_GENAU.format(new
		// Date(this.convertAbsoluteEchtZeit(naechsteAbfrageLve))));

		final int index = ((paramIndex - 1) + paramLen) % paramLen;

		// Zyklus ist genau der anzahl der Stunden
		// Assert.assertTrue("Stunden", abfrageIntervallLveNeu == 0);
		// Assert.assertTrue("Stunden", abfrageIntervallLve == lveZyklus[index]
		// * AbfrPufferKonstanten.STUNDE_IN_MS);

		// Offset ist max 1000 milisekunden anders
		final Calendar c = Calendar.getInstance();
		c.setTimeInMillis(this.convertAbsoluteEchtZeit(getNaechsteAbfrageLve()));
		final long offset = (c.get(Calendar.MINUTE) * AbfrPufferKonstanten.MINUTE_IN_MS)
				+ (c.get(Calendar.SECOND) * 1000) + c.get(Calendar.MILLISECOND);
		final long diff = Math.abs(offset - (lveOffset[index] * AbfrPufferKonstanten.MINUTE_IN_MS));

		Assert.assertTrue(getNaechsteAbfrageLveNeu() == 0);
		Assert.assertTrue("Offset diff = " + diff, diff < 1000);
	}

	@Override
	void generiereRegelmaesigeAxlAbfragen() {
		super.generiereRegelmaesigeAxlAbfragen();

		// System.out.println("## AXL SCHEDULER @"
		// + DUAKonstanten.ZEIT_FORMAT_GENAU.format(new
		// Date(System.currentTimeMillis())) + ", Abfrageversuch: "
		// + DUAKonstanten.ZEIT_FORMAT_GENAU.format(new
		// Date(naechsteAbfrageAxl)) + ", Virtuell: "
		// + DUAKonstanten.ZEIT_FORMAT_GENAU.format(new
		// Date(this.convertAbsoluteEchtZeit(naechsteAbfrageAxl))));

		final int index = ((paramIndex - 1) + paramLen) % paramLen;

		// Zyklus ist genau der anzahl der Stunden
		// Assert.assertTrue("Stunden1", abfrageIntervallAxlNeu == 0);
		// Assert.assertTrue("Stunden2", abfrageIntervallAxl == axlZyklus[index]
		// * AbfrPufferKonstanten.STUNDE_IN_MS);

		// Offset ist max 1000 milisekunden anders
		final Calendar c = Calendar.getInstance();
		c.setTimeInMillis(this.convertAbsoluteEchtZeit(getNaechsteAbfrageAxl()));
		final long offset = (c.get(Calendar.MINUTE) * AbfrPufferKonstanten.MINUTE_IN_MS)
				+ (c.get(Calendar.SECOND) * 1000) + c.get(Calendar.MILLISECOND);
		final long diff = Math.abs(offset - (axlOffset[index] * AbfrPufferKonstanten.MINUTE_IN_MS));

		Assert.assertTrue(getNaechsteAbfrageAxlNeu() == 0);
		Assert.assertTrue("Offset diff = " + diff, diff < 1000);
	}

	/**
	 * Ende des Tests.
	 */
	public synchronized void ende() {
		this.notify();
	}

	@Override
	public void initLveAxlVerwaltung() {
		setLveDeVerwaltung(new LveDeVerwaltungTester(this, getStationen()));
		setAxlDeVerwaltung(new AxlDeVerwaltungTester(this, getStationen()));
		setLzdAggregation(new AggregatorVerwaltungTester());
	}

	@Override
	public long currentTimeMillis() {
		return ZeitSimulator.getTimeMillis();
	}

	@Override
	public long convertZeitIntervall(final long zeitIntervall) {
		return ZeitSimulator.getVirtuelleZeitIntervallAlsEcht(zeitIntervall);
	}

	@Override
	public long convertAbsoluteEchtZeit(final long zeit) {
		return ZeitSimulator.getEchteZeitAlsVirtuell(zeit);
	}

	@Override
	public long convertAbsoluteVirtuelleZeit(final long zeit) {
		return ZeitSimulator.getVirtuelleZeitAlsEcht(zeit);
	}

	protected long[] getAxlOffset() {
		return axlOffset;
	}

	protected Collection<SteuerModulSim> getSteuerModuleSim() {
		return steuerModuleSim;
	}

	protected int getParamLen() {
		return paramLen;
	}

	protected long[] getAxlZyklus() {
		return axlZyklus;
	}

	protected long[] getLveOffset() {
		return lveOffset;
	}

	protected long[] getLveZyklus() {
		return lveZyklus;
	}
}
