/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.util.Collection;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.bitctrl.dua.DUAKonstanten;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;

/**
 * Simulation eines Datenendgeraets
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public abstract class DeSim implements ClientReceiverInterface, ClientSenderInterface {

	/**
	 * Der Pid des DEs
	 */
	private final String dePid;

	/**
	 * FahlerMeldungen ueber dem Zustand des DEs
	 */
	private static DataDescription ddGloFehler;
	/**
	 * Datenbeschreibung von Archrivparemterierung
	 */
	private static DataDescription ddArchivParam;
	/**
	 * Ob man sich unmelden soll fuer GLOFEHLER Telgramme
	 */
	private boolean anmeldeGloFehler;
	/**
	 * Verbindung zum Datenverteiler
	 */
	private static ClientDavInterface dav;
	/**
	 * Ob man Online ist
	 */
	private boolean online = true;
	/**
	 * Ob man gar nich antwortet
	 */
	private boolean ohneAntwort;
	/**
	 * Ob man archiviert wird
	 */
	private boolean archivieren = true;
	/**
	 * Zeitstempel des letzten Abgefragten Datensatzes
	 */
	private long letzterZeitStempellGeschickt = AbfrPufferKonstanten.ZEIT_BEGINN;
	/**
	 * Zeitstempel des aeltesten Datensatzes dem man noch abschicken kann
	 */
	private long letzterDatenSatzVorhanden = AbfrPufferKonstanten.ZEIT_BEGINN;

	protected DeSim(final String dePid) {
		super();
		this.dePid = dePid;
	}

	/**
	 * Setzt den Zeitstempel des aeltesten DS, dem man noch abschicken kann
	 *
	 * @param letzterZeitStempellGeschickt
	 */
	public void setAeltesterZeitStempellVorhanden(final long letzterZeitStempellGeschickt) {
		this.letzterZeitStempellGeschickt = letzterZeitStempellGeschickt;
		this.letzterDatenSatzVorhanden = letzterZeitStempellGeschickt;
	}

	/**
	 * Initialisiert das DE
	 *
	 * @param connection
	 *            verbindung zum DAV
	 * @throws Exception
	 *             wird beim Fehler bei Anmeldung geworfen
	 */
	public void initialize(final ClientDavInterface connection) throws Exception {
		DeSim.dav = connection;
		// Globale Fehler Senden
		DeSim.ddGloFehler = erzeugeDatenBeschreibung(AbfrPufferKonstanten.ATG_TLS_GLODEFEHLER,
				AbfrPufferKonstanten.ASP_TLS_ANTWORT, (short) 0);
		DeSim.dav.subscribeSender(this, DeSim.dav.getDataModel().getObject(dePid), DeSim.ddGloFehler,
				SenderRole.source());
		anmeldeGloFehler = true;
		if (!SteuerModulSim.isAchivParametrierungAngemeldet()) {
			// ArchivParameter Senden
			DeSim.ddArchivParam = erzeugeDatenBeschreibung(AbfrPufferKonstanten.ATG_ARCHIV,
					AbfrPufferKonstanten.ASP_PARAM_VORGABE, (short) 0);
			DeSim.dav.subscribeSender(this, DeSim.dav.getDataModel().getConfigurationAuthority(), DeSim.ddArchivParam,
					SenderRole.sender());
			SteuerModulSim.setAchivParametrierungAngemeldet(true);
		}

	}

	/**
	 * Paramteriert dem Archiv, aendert sein status, durch eine Statische
	 * Methode fragt alle DEs nach, und sendet einen DS mit Einstellungen von
	 * allen DEs
	 *
	 * @param archivieren
	 */
	public void parametriereArchiv(final boolean archivieren) {
		// if(this.archivieren == archivieren) return;
		this.archivieren = archivieren;
		// System.out.format("%-60s PARAMETRIERUNG GEAENDERT %b\n", dePid,
		// archivieren );
		DeSim.sendeArchivParametrierung();
	}

	/**
	 * Paramteriert den Archiv, damit seine Einstellungen des Archivationstatus
	 * dem Model entschprechen
	 */
	public static void sendeArchivParametrierung() {
		final Data datensatz = DeSim.dav
				.createData(DeSim.dav.getDataModel().getAttributeGroup(AbfrPufferKonstanten.ATG_ARCHIV));
		datensatz.getArray("ParameterSatz").setLength(1);
		final Data.Array datenSpez = datensatz.getArray("ParameterSatz").getItem(0).getArray("DatenSpezifikation");
		datenSpez.setLength(1);
		final Data.Array objekte = datenSpez.getItem(0).getArray("Objekt");
		final Data.Array atg = datenSpez.getItem(0).getArray("AttributGruppe");
		atg.setLength(2);
		atg.asReferenceArray().getReferenceValue(0)
		.setSystemObject(DeSim.dav.getDataModel().getAttributeGroup(DUAKonstanten.ATG_LZD));
		atg.asReferenceArray().getReferenceValue(1).setSystemObject(
				DeSim.dav.getDataModel().getAttributeGroup(AbfrPufferKonstanten.ATG_TLS_AXLERGEBNIS_MELDUNG));

		final Collection<DeSim> deList = SteuerModulSim.getAllDes();
		objekte.asReferenceArray().setLength(deList.size());
		int i = 0;
		for (final DeSim de : deList) {
			if (de instanceof AxlDeKanal255Sim) {
				continue;
			} else if (de instanceof LveDeKanal255Sim) {
				continue;
			} else if (de instanceof AxlDeSim) {
				final AxlDeSim lde = (AxlDeSim) de;
				if (lde.isArchivieren()) {
					objekte.asReferenceArray().getReferenceValue(i++)
					.setSystemObject(DeSim.dav.getDataModel().getObject(lde.getMsPid()));
				}
			} else if (de instanceof LveDeSim) {
				final LveDeSim lde = (LveDeSim) de;
				if (lde.isArchivieren()) {
					objekte.asReferenceArray().getReferenceValue(i++)
					.setSystemObject(DeSim.dav.getDataModel().getObject(lde.getFsPid()));
				}
			}
		}
		objekte.asReferenceArray().setLength(i);

		final Data einstell = datensatz.getArray("ParameterSatz").getItem(0).getItem("Einstellungen");
		einstell.getItem("Archivieren").asUnscaledValue().set(1);

		final long zeit = ZeitSimulator.getTimeMillis();
		final ResultData resDatei = new ResultData(DeSim.dav.getDataModel().getConfigurationAuthority(),
				DeSim.ddArchivParam, zeit, datensatz);

		try {
			DeSim.dav.sendData(resDatei);
		} catch (final Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Bearbeitet die Aufrufargumente
	 */
	public void parseArguments(final ArgumentList argumentList) throws Exception {
	}

	@Override
	public void update(final ResultData[] results) {
	}

	@Override
	public void dataRequest(final SystemObject object, final DataDescription dataDescription, final byte state) {
		if (ohneAntwort) {
			return;
		}
		if (dataDescription.getAttributeGroup().getPid().equals(AbfrPufferKonstanten.ATG_TLS_GLODEFEHLER)
				&& (state == ClientSenderInterface.START_SENDING)
				&& object.equals(DeSim.dav.getDataModel().getObject(dePid))) {
			sendeTlsGloFehler(!online);
		} else if (dataDescription.getAttributeGroup().getPid().equals(AbfrPufferKonstanten.ATG_TLS_GLODEFEHLER)
				&& (state == ClientSenderInterface.STOP_SENDING)
				&& object.equals(DeSim.dav.getDataModel().getObject(dePid))) {
			sendeEnde();
		}
	}

	@Override
	public boolean isRequestSupported(final SystemObject object, final DataDescription dataDescription) {
		return true;
	}

	/**
	 * Wenn man Sendung beenden soll
	 */
	public void sendeEnde() {
		final long jetzt = ZeitSimulator.getTimeMillis();
		final ResultData resData = new ResultData(DeSim.dav.getDataModel().getObject(dePid), DeSim.ddGloFehler, jetzt,
				null, false, DataState.NO_DATA);
		try {
			DeSim.dav.sendData(resData);
		} catch (final Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Sendet einen TLSGLoFehler Telegramm
	 *
	 * @param fehler
	 *            Ob das DE fehlerhaft ist
	 */
	public void sendeTlsGloFehler(final boolean fehler) {
		final Data data = DeSim.dav
				.createData(DeSim.dav.getDataModel().getAttributeGroup(AbfrPufferKonstanten.ATG_TLS_GLODEFEHLER));
		if (fehler) {
			data.getUnscaledValue(AbfrPufferKonstanten.ATR_DE_FEHLERSTATUS).set(1);
		} else {
			data.getUnscaledValue(AbfrPufferKonstanten.ATR_DE_FEHLERSTATUS).set(0);
		}

		data.getUnscaledValue(AbfrPufferKonstanten.ATR_DE_KANALSTATUS).set(0);
		data.getUnscaledValue(AbfrPufferKonstanten.ATR_DE_PROJEKTSTATUS).set(0);
		data.getUnscaledValue(AbfrPufferKonstanten.ATR_HERSTELLERDEFCODE).set(0);
		data.getUnscaledValue(AbfrPufferKonstanten.ATR_HERSTELLER).set(0);

		final long jetzt = ZeitSimulator.getTimeMillis();

		final ResultData resData = new ResultData(DeSim.dav.getDataModel().getObject(dePid), DeSim.ddGloFehler, jetzt,
				data);
		try {
			DeSim.dav.sendData(resData);
		} catch (final Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * De geht online
	 *
	 * @param datenVorhanden
	 *            true, wenn die Daten waehrend des offine status nicht verloren
	 *            worden sind
	 */
	public void goOnline(final boolean datenVorhanden) {
		this.ohneAntwort = false;
		this.online = true;
		if (!datenVorhanden) {
			letzterDatenSatzVorhanden = ZeitSimulator.getTimeMillis();
		}
		sendeTlsGloFehler(false);
	}

	/**
	 * De geht offline
	 */
	public void goOffline() {
		this.online = false;
		sendeTlsGloFehler(true);
	}

	/**
	 * De geht offline ohne zu antworten oder benachrichtigen
	 */
	public void goOfflineOhneMeldung() {
		this.ohneAntwort = true;
	}

	/**
	 * Erfragt ob das DE online ist
	 *
	 * @return true, wenn online
	 */
	public boolean isOnline() {
		return !ohneAntwort && online;
	}

	/**
	 * Das De flackert, d.h. sendet 10 mal nacheinander eine Statusaenderung
	 */
	public void flackern() {
		for (int i = 0; i < 10; i++) {
			goOffline();
			goOnline(true);
		}
	}

	/**
	 * Abmeldet sich vom DAV
	 */
	public void unsubscribe() {
		if (anmeldeGloFehler) {
			DeSim.dav.unsubscribeSender(this, DeSim.dav.getDataModel().getObject(dePid), DeSim.ddGloFehler);
		}
		if (!SteuerModulSim.isAchivParametrierungAngemeldet()) {
			DeSim.dav.unsubscribeSender(this, DeSim.dav.getDataModel().getConfigurationAuthority(),
					DeSim.ddArchivParam);
		}

	}

	/**
	 * Erzeugt eine DatenBeschreibung mit gegebenen Parameter als Strings
	 *
	 * @param atg
	 *            Atributgruppe
	 * @param asp
	 *            Aspekt
	 * @param var
	 *            Simulation Variant
	 * @return DataDescription mit gegebene parameter
	 */
	public DataDescription erzeugeDatenBeschreibung(final String atg, final String asp, final short var) {
		return new DataDescription(DeSim.dav.getDataModel().getAttributeGroup(atg),
				DeSim.dav.getDataModel().getAspect(asp), var);
	}

	/**
	 * Ergibt aus der Liste das De mit pid
	 *
	 * @param deListe
	 * @param pid
	 * @return Datenendgeraet
	 */
	static DeSim getDe(final Collection<DeSim> deListe, final String pid) {
		for (final DeSim de : deListe) {
			if (de.dePid.equals(pid)) {
				return de;
			}
		}
		return null;
	}

	/**
	 * Liefert den Pid
	 *
	 * @return Pid
	 */
	public String getDePid() {
		return dePid;
	}

	/**
	 * Erfragt ob das De zum Archivieren eingestellt ist
	 *
	 * @return true, wenn Daten archiviert werden
	 */
	public boolean isArchivieren() {
		return archivieren;
	}

	@Override
	public String toString() {
		return dePid;
	}

	protected boolean isOhneAntwort() {
		return ohneAntwort;
	}

	protected long getLetzterZeitStempellGeschickt() {
		return letzterZeitStempellGeschickt;
	}

	protected void setLetzterZeitStempellGeschickt(final long letzterZeitStempellGeschickt) {
		this.letzterZeitStempellGeschickt = letzterZeitStempellGeschickt;
	}

	protected long getLetzterDatenSatzVorhanden() {
		return letzterDatenSatzVorhanden;
	}

	protected void setLetzterDatenSatzVorhanden(final long letzterDatenSatzVorhanden) {
		this.letzterDatenSatzVorhanden = letzterDatenSatzVorhanden;
	}

	protected static ClientDavInterface getDav() {
		return DeSim.dav;
	}

	protected static void setDav(final ClientDavInterface dav) {
		DeSim.dav = dav;
	}

	protected static DataDescription getDdArchivParam() {
		return DeSim.ddArchivParam;
	}

}
