/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.text.DateFormat;
import java.text.SimpleDateFormat;

import org.junit.Test;

import com.bitctrl.Constants;

import de.bsvrz.dua.abfrpuffer.Ereignis.EreignissTyp;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.bitctrl.dua.DUAKonstanten;

/**
 * Auslagerung einer Testroutine zum einzeln startbaren Test, damit zwischen den
 * Tests das Archiv geleert werden kann
 *
 * @author BitCtrl Systems GmbH, Thierfelder
 *
 */
public class NegativeQuittungTester extends AbfragePufferDatenTester {

	/**
	 * Testfall, das SystemDe kann die Anfragen nicht verstehen, antwortete mit
	 * Negativer Quittung
	 */
	@Override
	@Test
	public synchronized void runNegativeQuittungTest() {
		try {
			final DateFormat datumFormat = new SimpleDateFormat(
					DUAKonstanten.ZEIT_FORMAT_GENAU_STR);

			long zeit = datumFormat.parse("2006.01.01 20:00:00.000").getTime();
			SteuerModulSim.setAeltesterLveZeitStempellVorhanden(
					getSteuerModuleSim(), zeit);

			zeit = datumFormat.parse("2006.01.01 20:00:00.000").getTime();
			SteuerModulSim.setAeltesterAxlZeitStempellVorhanden(
					getSteuerModuleSim(), zeit);

			ZeitSimulator.start("2006.01.03 12:00:00.000", 5000, 8000);
			StandardApplicationRunner.run(this, Verbindung.CON_DATA.clone());

			ZeitSimulator
					.schedule(
							new Ereignis(
									SteuerModulSim
											.getDe("testLVE.SM_1004.EAK1.LveDeSammelkanal_1"),
									EreignissTyp.EREIGNIS_DE_KANN_ABFRAGE_NICHT_VERSTEHEN),
							"2006.01.03 17:30:00.000");

			ZeitSimulator.schedule(
					new Ereignis(SteuerModulSim
							.getDe("testLVE.SM_1004.EAK1.DeLve_1"),
							EreignissTyp.EREIGNIS_SM_OFFLINE),
					"2006.01.04 10:30:00.000");

			ZeitSimulator.schedule(
					new Ereignis(SteuerModulSim
							.getDe("testLVE.SM_1004.EAK1.DeLve_1"),
							EreignissTyp.EREIGNIS_DE_GEHT_ONLINE_MIT_DATEN),
					"2006.01.04 11:30:00.000");

			ZeitSimulator
					.schedule(
							new Ereignis(
									SteuerModulSim
											.getDe("testLVE.SM_1004.EAK1.LveDeSammelkanal_1"),
									EreignissTyp.EREIGNIS_DE_KANN_ABFRAGE_WIEDER_VERSTEHEN),
							"2006.01.04 12:30:00.000");

			ZeitSimulator.schedule(new Ereignis(this,
					EreignissTyp.EREIGNIS_KOSISTENZTEST),
					"2006.01.04 23:30:00.000");

			ZeitSimulator.schedule(new Ereignis(this,
					EreignissTyp.EREIGNIS_SIMULATION_ENDE),
					"2006.01.05 01:00:00.000");
			this.wait();

		} catch (final Exception e) {
			getDav().disconnect(true,
					"Fehler bei Test: runNegativeQuittungTest()");
			System.err.println(e.getMessage());
			e.printStackTrace();
		}

		getDav().disconnect(false, Constants.EMPTY_STRING);
	}

}