/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer.aggreg;

import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.LinkedList;

import org.junit.Ignore;
import org.junit.Test;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dua.abfrpuffer.Verbindung;
import de.bsvrz.dua.abfrpuffer.dtimport.TestFSImporterGeschwKlasse;
import de.bsvrz.sys.funclib.bitctrl.dua.DUAKonstanten;

/**
 * Hauptklasse des Aggregationstests, die Tesfaelle sind als JUnit Methoden
 * implementiert.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
@Ignore("Datenverteilerverbindung klären")
public class AggregationTest {

	/**
	 * Pfade zu die Quelle der Datensaetzen, eine CSV Tabelle.
	 */
	protected static final String QUELLE = Verbindung.TEST_DATEN_WURZEL + "Aggreg-TEST.csv";

	/**
	 * Liste der Fahrstreifen die zu die aggregierte LZZ gehoeren.
	 */
	protected static final String[] FS_NAME = new String[] { "fs.test.1.1.1", "fs.test.1.1.2", "fs.test.1.1.3",
			"fs.test.1.2.1", "fs.test.1.2.2", "fs.test.1.2.3" };
	/**
	 * Anzahl der FS in der vorherigen Liste.
	 */
	private final int fsAnzahl = AggregationTest.FS_NAME.length;

	/**
	 * Verbindung zum DAV.
	 */
	private ClientDavConnection oClientDavConnection;
	/**
	 * Paramteres zur Verbindung.
	 */
	private ClientDavParameters parameters;
	/**
	 * CSV Importer fuer FahrStreifen.
	 */
	private TestFSImporterGeschwKlasse oFSImporter;

	/**
	 * Testet die Aggregation mit einer vorher ausgerechneten Tabelle
	 * Aggreg-TEST.csv.
	 */
	@Test
	public void test1() {

		try {
			parameters = new ClientDavParameters(Verbindung.CON_DATA.clone());

			oClientDavConnection = new ClientDavConnection(parameters);
			oClientDavConnection.connect();
			oClientDavConnection.login();

			final SystemObject lzz = oClientDavConnection.getDataModel().getObject("lzz.testLZZ.swe4.6.1");
			final SystemObject[] fs = new SystemObject[fsAnzahl];
			for (int i = 0; i < fsAnzahl; i++) {
				fs[i] = oClientDavConnection.getDataModel().getObject(AggregationTest.FS_NAME[i]);
			}

			oFSImporter = new TestFSImporterGeschwKlasse(oClientDavConnection, AggregationTest.QUELLE);

			final AttributeGroup atg = oClientDavConnection.getDataModel().getAttributeGroup(DUAKonstanten.ATG_LZD);
			Data data;

			final Calendar cal = Calendar.getInstance();
			cal.clear();
			cal.set(2007, 7, 18, 14, 0);
			final Date zeitStempel = cal.getTime();

			final Collection<SystemObject> soColl = new LinkedList<>();
			soColl.add(lzz);

			final FahrzeugAnzahlTester agg1 = new FahrzeugAnzahlTester();
			final GeschwindigkeitTester agg2 = new GeschwindigkeitTester();
			final SpizenStundeTester agg3 = new SpizenStundeTester();
			final VxxTester agg4 = new VxxTester(85);

			final AttributeGroup atg1 = oClientDavConnection.getDataModel()
					.getAttributeGroup("atg.langZeitDatenFahrzeugAnzahl");
			final AttributeGroup atg2 = oClientDavConnection.getDataModel()
					.getAttributeGroup("atg.langZeitDatenGeschwindigkeit");
			final AttributeGroup atg3 = oClientDavConnection.getDataModel()
					.getAttributeGroup("atg.langZeitDatenSpitzenStunde");
			// AttributeGroup atg4 =
			// oClientDavConnection.getDataModel().getAttributeGroup("atg.langZeitDatenV85");

			agg1.init(oClientDavConnection.getDataModel().getAspect("asp.aggregationsIntervallTag"),
					oClientDavConnection, soColl);
			agg2.init(oClientDavConnection.getDataModel().getAspect("asp.aggregationsIntervallTag"),
					oClientDavConnection, soColl);
			agg3.init(oClientDavConnection.getDataModel().getAspect("asp.aggregationsIntervallTag"),
					oClientDavConnection, soColl);
			agg4.init(oClientDavConnection.getDataModel().getAspect("asp.aggregationsIntervallTag"),
					oClientDavConnection, soColl);

			for (int i = 0; i < 10; i++) {
				data = oFSImporter.getNaechstenDatensatz(atg, false);
				agg1.bearbeiteDatenSatz(lzz, data, zeitStempel.getTime());
				agg2.bearbeiteDatenSatz(lzz, data, zeitStempel.getTime());
				agg3.bearbeiteDatenSatz(lzz, data, zeitStempel.getTime());
				agg4.bearbeiteDatenSatz(lzz, data, zeitStempel.getTime());
			}

			// 11. Datensatz ist die FahrzeugAnzahSumme
			agg1.setReferenzWert(oFSImporter.getNaechstenAggregDatensatz(atg1, false));
			// 12. Datensatz ist der Mittelwert der Geschwindigkeit
			agg2.setReferenzWert(oFSImporter.getNaechstenAggregDatensatz(atg2, false));
			// 13. DatenSatz sind die SpizenWerte
			agg3.setReferenzWert(oFSImporter.getNaechstenSpitzenStundeDatensatz(atg3, false));
			// 14. DatenSatz sind die durschschnittwerte von V85
			// Geschwindigkeiten
			// agg4.setReferenzWert(oFSImporter.getNaechstenAggregDatensatz(atg4,
			// false));

			System.out.println("\n\nTest1 - testet, ob die aggregierten Werte korrekt sind");
			agg1.erzeugeErgebniss(lzz, zeitStempel.getTime(), zeitStempel.getTime(), true);
			agg2.erzeugeErgebniss(lzz, zeitStempel.getTime(), zeitStempel.getTime(), true);
			agg3.erzeugeErgebniss(lzz, zeitStempel.getTime(), zeitStempel.getTime(), true);
			// agg4.erzeugeErgebniss(LZZ, zeitStempel.getTime(),
			// zeitStempel.getTime(), true);

			oClientDavConnection.disconnect(false, "");
			parameters = null;
			oClientDavConnection = null;

		} catch (final Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Testet die Berechnungen mit leeren Datensaetzen.
	 */
	@Test
	public void test2() {

		try {
			parameters = new ClientDavParameters(Verbindung.CON_DATA.clone());

			oClientDavConnection = new ClientDavConnection(parameters);
			oClientDavConnection.connect();
			oClientDavConnection.login();

			final SystemObject lzz = oClientDavConnection.getDataModel().getObject("lzz.testLZZ.swe4.6.1");
			final SystemObject[] fs = new SystemObject[fsAnzahl];
			for (int i = 0; i < fsAnzahl; i++) {
				fs[i] = oClientDavConnection.getDataModel().getObject(AggregationTest.FS_NAME[i]);
			}

			final AttributeGroup atg = oClientDavConnection.getDataModel().getAttributeGroup(DUAKonstanten.ATG_LZD);
			Data data;

			final Calendar cal = Calendar.getInstance();
			cal.clear();
			cal.set(2007, 7, 18, 14, 0);
			final Date zeitStempel = cal.getTime();

			final Collection<SystemObject> soColl = new LinkedList<>();
			soColl.add(lzz);

			final FahrzeugAnzahlTester agg1 = new FahrzeugAnzahlTester();
			final GeschwindigkeitTester agg2 = new GeschwindigkeitTester();
			final SpizenStundeTester agg3 = new SpizenStundeTester();
			final VxxTester agg4 = new VxxTester(85);

			final AttributeGroup atg1 = oClientDavConnection.getDataModel()
					.getAttributeGroup("atg.langZeitDatenFahrzeugAnzahl");
			final AttributeGroup atg2 = oClientDavConnection.getDataModel()
					.getAttributeGroup("atg.langZeitDatenGeschwindigkeit");
			final AttributeGroup atg3 = oClientDavConnection.getDataModel()
					.getAttributeGroup("atg.langZeitDatenSpitzenStunde");
			final AttributeGroup atg4 = oClientDavConnection.getDataModel().getAttributeGroup("atg.langZeitDatenV85");

			agg1.init(oClientDavConnection.getDataModel().getAspect("asp.aggregationsIntervallTag"),
					oClientDavConnection, soColl);
			agg2.init(oClientDavConnection.getDataModel().getAspect("asp.aggregationsIntervallTag"),
					oClientDavConnection, soColl);
			agg3.init(oClientDavConnection.getDataModel().getAspect("asp.aggregationsIntervallTag"),
					oClientDavConnection, soColl);
			agg4.init(oClientDavConnection.getDataModel().getAspect("asp.aggregationsIntervallTag"),
					oClientDavConnection, soColl);

			data = oClientDavConnection.createData(atg);
			TestFSImporterGeschwKlasse.setLZDleer(data, false);

			for (int i = 0; i < 10; i++) {
				agg1.bearbeiteDatenSatz(lzz, data, zeitStempel.getTime());
				agg2.bearbeiteDatenSatz(lzz, data, zeitStempel.getTime());
				agg3.bearbeiteDatenSatz(lzz, data, zeitStempel.getTime());
				agg4.bearbeiteDatenSatz(lzz, data, zeitStempel.getTime());
			}

			data = oClientDavConnection.createData(atg1);
			agg1.setLeer(data);
			agg1.setReferenzWert(data);

			data = oClientDavConnection.createData(atg2);
			agg2.setLeer(data);
			agg2.setReferenzWert(data);

			data = oClientDavConnection.createData(atg3);
			agg3.setLeer(data);
			agg3.setReferenzWert(data);

			data = oClientDavConnection.createData(atg4);
			agg4.setLeer(data);
			agg4.setReferenzWert(data);

			System.out.println(
					"\n\nTest2 - erzeugt das Ergebniss, wobei alle eingekommene Datensaetze mit nicht ermittelbaren Werte eingekommen sind");
			agg1.erzeugeErgebniss(lzz, zeitStempel.getTime(), zeitStempel.getTime(), true);
			agg2.erzeugeErgebniss(lzz, zeitStempel.getTime(), zeitStempel.getTime(), true);
			agg3.erzeugeErgebniss(lzz, zeitStempel.getTime(), zeitStempel.getTime(), true);
			agg4.erzeugeErgebniss(lzz, zeitStempel.getTime(), zeitStempel.getTime(), true);

			oClientDavConnection.disconnect(false, "");
			parameters = null;
			oClientDavConnection = null;

		} catch (final Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Testet die Berechnungen ohne einkomemende Datensaetze.
	 */
	@Test
	public void test3() {

		try {
			parameters = new ClientDavParameters(Verbindung.CON_DATA.clone());

			oClientDavConnection = new ClientDavConnection(parameters);
			oClientDavConnection.connect();
			oClientDavConnection.login();

			final SystemObject lzz = oClientDavConnection.getDataModel().getObject("lzz.testLZZ.swe4.6.1");
			final SystemObject[] fs = new SystemObject[fsAnzahl];
			for (int i = 0; i < fsAnzahl; i++) {
				fs[i] = oClientDavConnection.getDataModel().getObject(AggregationTest.FS_NAME[i]);
			}

			Data data;

			final Calendar cal = Calendar.getInstance();
			cal.clear();
			cal.set(2007, 7, 18, 14, 0);
			final Date zeitStempel = cal.getTime();

			final Collection<SystemObject> soColl = new LinkedList<>();
			soColl.add(lzz);

			final FahrzeugAnzahlTester agg1 = new FahrzeugAnzahlTester();
			final GeschwindigkeitTester agg2 = new GeschwindigkeitTester();
			final SpizenStundeTester agg3 = new SpizenStundeTester();
			final VxxTester agg4 = new VxxTester(85);

			final AttributeGroup atg1 = oClientDavConnection.getDataModel()
					.getAttributeGroup("atg.langZeitDatenFahrzeugAnzahl");
			final AttributeGroup atg2 = oClientDavConnection.getDataModel()
					.getAttributeGroup("atg.langZeitDatenGeschwindigkeit");
			final AttributeGroup atg3 = oClientDavConnection.getDataModel()
					.getAttributeGroup("atg.langZeitDatenSpitzenStunde");
			final AttributeGroup atg4 = oClientDavConnection.getDataModel().getAttributeGroup("atg.langZeitDatenV85");

			agg1.init(oClientDavConnection.getDataModel().getAspect("asp.aggregationsIntervallTag"),
					oClientDavConnection, soColl);
			agg2.init(oClientDavConnection.getDataModel().getAspect("asp.aggregationsIntervallTag"),
					oClientDavConnection, soColl);
			agg3.init(oClientDavConnection.getDataModel().getAspect("asp.aggregationsIntervallTag"),
					oClientDavConnection, soColl);
			agg4.init(oClientDavConnection.getDataModel().getAspect("asp.aggregationsIntervallTag"),
					oClientDavConnection, soColl);

			data = oClientDavConnection.createData(atg1);
			agg1.setLeer(data);
			agg1.setReferenzWert(data);

			data = oClientDavConnection.createData(atg2);
			agg2.setLeer(data);
			agg2.setReferenzWert(data);

			data = oClientDavConnection.createData(atg3);
			agg3.setLeer(data);
			agg3.setReferenzWert(data);

			data = oClientDavConnection.createData(atg4);
			agg4.setLeer(data);
			agg4.setReferenzWert(data);

			System.out.println("\n\nTest3 - erzeugt ergebniss ohne eingekommene Datensaetze");
			agg1.erzeugeErgebniss(lzz, zeitStempel.getTime(), zeitStempel.getTime(), true);
			agg2.erzeugeErgebniss(lzz, zeitStempel.getTime(), zeitStempel.getTime(), true);
			agg3.erzeugeErgebniss(lzz, zeitStempel.getTime(), zeitStempel.getTime(), true);
			agg4.erzeugeErgebniss(lzz, zeitStempel.getTime(), zeitStempel.getTime(), true);

			oClientDavConnection.disconnect(false, "");
			parameters = null;
			oClientDavConnection = null;

		} catch (final Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}

	}
}
