/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer.dtimport;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dua.abfrpuffer.AbfrPufferKonstanten;
import de.bsvrz.dua.abfrpuffer.Verbindung;
import de.bsvrz.sys.funclib.bitctrl.dua.DUAKonstanten;

/**
 * Liest die Ausgangsdaten eines Fahrstreifens mit Hilfe der Klasse
 * TestFSImporterGewschwklasse ein und sendet diese in dem DAV.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class ArchiveImporter implements ClientSenderInterface {

	/**
	 * Quelle der LZD, eine CSV tabelle.
	 */
	private static String quelle = Verbindung.TEST_DATEN_WURZEL + new String("TFahrStreifenWerte-Steigend.csv");

	/**
	 * Ob die Daten als nachgeliefert gesendet werden sollen.
	 */
	private static boolean nachgeliefert = true;
	/**
	 * Ob man auch Geschwindigkeitsdaten erzeugen soll.
	 */
	private static boolean geschwDatenErzeugen;
	/**
	 * Verbindung zum DAV.
	 */
	private static ClientDavConnection oClientDavConnection;
	/**
	 * Paramteres zur Verbindung.
	 */
	private static ClientDavParameters parameters;
	/**
	 * CSV Importer fuer FahrStreifen.
	 */
	private static TestFSImporterGeschwKlasse oFSImporter;
	/**
	 * Sende-Datenbeschreibung für LZD.
	 */
	private static DataDescription ddLzdSend;

	/**
	 * Hauptmethode.
	 *
	 * @param args
	 *            Aufrufsparameter
	 */
	public static void main(final String[] args) {
		final ArchiveImporter fsim = new ArchiveImporter();
		fsim.run(args);
	}

	/**
	 * Erstelllet eine Verbindung mit dem DAV, args sind die Parametern, Liest
	 * die Daten vom CSV und schickt sie ab zum DAV.
	 *
	 * @param args
	 *            Argumente für das ClientDavConnection
	 *
	 */
	public void run(final String[] args) {
		System.out.println("Verbindung in Bearbeitung");

		try {

			final String[] fsName = new String[] {
					// "fs.test.1.1.1",
					// "fs.test.1.1.2",
					// "fs.test.1.1.3",
					"fs.test.1.2.1", "fs.test.1.2.2", "fs.test.1.2.3",
					// "fs.test.2.1.1",
					// "fs.test.2.1.2",
					// "fs.test.2.1.3",
					// "fs.test.2.2.1",
					// "fs.test.2.2.2",
					// "fs.test.2.2.3",
					// "fs.test.3.1.1",
					// "fs.test.3.1.2",
					// "fs.test.3.1.3",
					// "fs.test.3.2.1",
					// "fs.test.3.2.2",
					// "fs.test.3.2.3",
			};

			final int fsAnzahl = fsName.length;

			ArchiveImporter.parameters = new ClientDavParameters(args);

			ArchiveImporter.oClientDavConnection = new ClientDavConnection(ArchiveImporter.parameters);
			ArchiveImporter.oClientDavConnection.connect();
			ArchiveImporter.oClientDavConnection.login();

			final SystemObject[] fs = new SystemObject[fsAnzahl];
			for (int i = 0; i < fsAnzahl; i++) {
				fs[i] = ArchiveImporter.oClientDavConnection.getDataModel().getObject(fsName[i]);
			}

			ArchiveImporter.oFSImporter = new TestFSImporterGeschwKlasse(ArchiveImporter.oClientDavConnection,
					ArchiveImporter.quelle);
			ArchiveImporter.ddLzdSend = new DataDescription(
					ArchiveImporter.oClientDavConnection.getDataModel().getAttributeGroup(DUAKonstanten.ATG_LZD),
					ArchiveImporter.oClientDavConnection.getDataModel().getAspect(DUAKonstanten.ASP_EXTERNE_ERFASSUNG));

			final AttributeGroup atg = ArchiveImporter.oClientDavConnection.getDataModel()
					.getAttributeGroup(DUAKonstanten.ATG_LZD);
			ArchiveImporter.oClientDavConnection.subscribeSender(this, fs, ArchiveImporter.ddLzdSend,
					SenderRole.source());
			final Data data;
			final SimpleDateFormat df = new SimpleDateFormat("HH:mm:ss yyyy.MM.dd");

			final Date zeitStempel;
			final Calendar cal = Calendar.getInstance();
			cal.clear();
			cal.set(2007, 2, 20, 14, 0);

			final Calendar calendar = Calendar.getInstance();
			calendar.clear();
			calendar.set(2007, 2, 3, 10, 0);
			zeitStempel = calendar.getTime();

			data = ArchiveImporter.oFSImporter.getNaechstenDatensatz(atg, ArchiveImporter.geschwDatenErzeugen);
			data.getTimeValue("T").setMillis(AbfrPufferKonstanten.STUNDE_IN_MS);
			if (Verbindung.VERBOSE) {
				System.out.println(data);
			}

			int j = 0;
			while (data != null) {
				final ResultData resultData = new ResultData(fs[(j) % fsAnzahl], ArchiveImporter.ddLzdSend,
						zeitStempel.getTime(), data, ArchiveImporter.nachgeliefert);
				ArchiveImporter.oClientDavConnection.sendData(resultData);
				System.out.println("Datum fuer " + fsName[j % fsAnzahl] + " geschickt mit ZeitStempel: "
						+ df.format(zeitStempel) + " Nachgeliefert = " + ArchiveImporter.nachgeliefert);
				// data = oFSImporter.getNaechstenDatensatz(atg,
				// geschwDatenErzeugen);

				if ((j % fsAnzahl) == (fsAnzahl - 1)) {
					zeitStempel.setTime(AbfrPufferKonstanten.STUNDE_IN_MS + zeitStempel.getTime());
				}
				if ((j % fsAnzahl) == (fsAnzahl - 1)) {
					Thread.sleep(1000);
				}
				j++;

				// TODO break eingeführt, ansonsten Endlosschleife: Funktion
				// klären!
				break;

			}
			ArchiveImporter.oClientDavConnection.disconnect(false, "");
		} catch (final Exception e) {
			System.err.println("Fehler:");
			e.printStackTrace();
			System.exit(1);
		}

		System.exit(0);
	}

	@Override
	public void dataRequest(final SystemObject object, final DataDescription dataDescription, final byte state) {
		// VOID
	}

	@Override
	public boolean isRequestSupported(final SystemObject object, final DataDescription dataDescription) {
		return false;
	}
}
